package cc.fascinated.bat.features.spotify.command;

import cc.fascinated.bat.command.BatSubCommand;
import cc.fascinated.bat.command.CommandInfo;
import cc.fascinated.bat.common.EmbedUtils;
import cc.fascinated.bat.features.spotify.SpotifyFeature;
import cc.fascinated.bat.features.spotify.profile.SpotifyProfile;
import cc.fascinated.bat.model.BatGuild;
import cc.fascinated.bat.model.BatUser;
import cc.fascinated.bat.service.SpotifyService;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.dv8tion.jda.api.entities.Member;
import net.dv8tion.jda.api.entities.channel.middleman.MessageChannel;
import net.dv8tion.jda.api.interactions.commands.SlashCommandInteraction;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import se.michaelthelin.spotify.model_objects.miscellaneous.CurrentlyPlaying;
import se.michaelthelin.spotify.model_objects.specification.AlbumSimplified;
import se.michaelthelin.spotify.model_objects.specification.Image;
import se.michaelthelin.spotify.model_objects.specification.Track;

/**
 * @author Fascinated (fascinated7)
 */
@Component
@Log4j2
@CommandInfo(name = "current", description = "Gets the currently playing Spotify track")
public class CurrentSubCommand extends BatSubCommand {
    private final SpotifyService spotifyService;

    @Autowired
    public CurrentSubCommand(@NonNull SpotifyService spotifyService) {
        this.spotifyService = spotifyService;
    }

    @Override
    public void execute(BatGuild guild, @NonNull BatUser user, @NonNull MessageChannel channel, Member member, @NonNull SlashCommandInteraction interaction) {
        SpotifyProfile profile = user.getProfile(SpotifyProfile.class);
        if (!profile.hasLinkedAccount()) {
            interaction.replyEmbeds(SpotifyFeature.linkAccountEmbed()).queue();
            return;
        }

        if (!spotifyService.hasTrackPlaying(user)) {
            interaction.replyEmbeds(EmbedUtils.genericEmbed()
                            .setDescription("You are not currently playing a track.")
                            .build())
                    .queue();
            return;
        }
        CurrentlyPlaying currentlyPlaying = spotifyService.getCurrentlyPlayingTrack(user);
        Track track = (Track) currentlyPlaying.getItem();
        AlbumSimplified album = track.getAlbum();
        String trackUrl = "https://open.spotify.com/track/" + track.getId();
        String albumUrl = "https://open.spotify.com/album/" + album.getId();

        String description =
                "➜ Song: **[%s](%s)**\n".formatted(track.getName(), trackUrl) +
                        "➜ Album: **[%s](%s)**\n".formatted(album.getName(), albumUrl) +
                        "➜ Position: %s\n".formatted(getFormattedTime(currentlyPlaying));

        Image albumCover = album.getImages()[0];
        interaction.replyEmbeds(EmbedUtils.genericEmbed()
                .setAuthor("Listening to %s".formatted(track.getName()), trackUrl)
                .setThumbnail(albumCover.getUrl())
                .setDescription(description)
                .build()).queue();
    }


    /**
     * Gets the formatted time of the currently playing track
     *
     * @param currentlyPlaying the currently playing track
     * @return the formatted time
     */
    private String getFormattedTime(@NonNull CurrentlyPlaying currentlyPlaying) {
        Track track = (Track) currentlyPlaying.getItem();
        int currentMinutes = currentlyPlaying.getProgress_ms() / 1000 / 60;
        int currentSeconds = currentlyPlaying.getProgress_ms() / 1000 % 60;
        int totalMinutes = track.getDurationMs() / 1000 / 60;
        int totalSeconds = track.getDurationMs() / 1000 % 60;

        return "`%02d:%02d`/`%02d:%02d`".formatted(currentMinutes, currentSeconds, totalMinutes, totalSeconds);
    }
}
